@@ -1,3 +1,251 @@
+=begin
+
+= fileutils.rb
+
+  Copyright (c) 2000-2002 Minero Aoki <aamine@loveruby.net>
+
+  This program is free software.
+  You can distribute/modify this program under the same terms of ruby.
+
+== module FileUtils
+
+The module which implements basic file operations.
+
+=== Module Functions
+
+--- FileUtils.cd( dir, *options )
+--- FileUtils.cd( dir, *options ) {|dir| .... }
+  Options: noop verbose
+
+  changes the current directory to the directory DIR.
+
+  If this method is called with block, resumes to the old
+  working directory after the block execution finished.
+
+    FileUtils.cd '/', :verbose   # chdir and report it
+
+--- FileUtils.uptodate?( newer, older_list, *options )
+  Options: verbose
+
+  returns true if NEWER is newer than all OLDER_LIST.
+  Non-exist files are older than any file.
+
+    FileUtils.newest? 'hello.o', 'hello.c', 'hello.h' or system 'make'
+
+--- FileUtils.mkdir( dir, *options )
+  Options: noop verbose
+
+  creates directorie(s) DIR.
+
+    FileUtils.mkdir 'test'
+    FileUtils.mkdir %w( tmp data )
+    FileUtils.mkdir 'notexist', :noop  # does not create really
+
+--- FileUtils.mkdir_p( dir, *options )
+  Options: noop verbose
+
+  makes dirctories DIR and all its parent directories.
+  For example,
+
+    FileUtils.mkdir_p '/usr/local/bin/ruby'
+  
+  causes to make following directories (if it does not exist).
+      * /usr
+      * /usr/local
+      * /usr/local/bin
+      * /usr/local/bin/ruby
+
+--- FileUtils.rmdir( dir, *options )
+  Options: noop, verbose
+
+  removes directory DIR.
+
+    FileUtils.rmdir 'somedir'
+    FileUtils.rmdir %w(somedir anydir otherdir)
+    # does not remove directory really, outputing message.
+    FileUtils.rmdir 'somedir', :verbose, :noop
+
+--- FileUtils.ln( old, new, *options )
+  Options: force noop verbose
+
+  creates a hard link NEW which points OLD.
+  If NEW already exists and it is a directory, creates a symbolic link NEW/OLD.
+  If NEW already exists and it is not a directory, raises Errno::EEXIST.
+  But if :force option is set, overwrite NEW.
+
+    FileUtils.ln 'gcc', 'cc', :verbose
+    FileUtils.ln '/usr/bin/emacs21', '/usr/bin/emacs'
+
+--- FileUtils.ln( list, destdir, *options )
+  Options: force noop verbose
+
+  creates hard links DESTDIR/LIST[0], DESTDIR/LIST[1], DESTDIR/LIST[2], ...
+  And each link points LIST[0], LIST[1], LIST[2], ...
+  If DESTDIR is not a directory, raises Errno::ENOTDIR.
+
+    include FileUtils
+    cd '/bin'
+    ln %w(cp mv mkdir), '/usr/bin'
+
+--- FileUtils.ln_s( old, new, *options )
+  Options: force noop verbose
+
+  creates a symbolic link NEW which points OLD.
+  If NEW already exists and it is a directory, creates a symbolic link NEW/OLD.
+  If NEW already exists and it is not a directory, raises Errno::EEXIST.
+  But if :force option is set, overwrite NEW.
+
+    FileUtils.ln_s '/usr/bin/ruby', '/usr/local/bin/ruby'
+    FileUtils.ln_s 'verylongsourcefilename.c', 'c', :force
+
+--- FileUtils.ln_s( list, destdir, *options )
+  Options: force noop verbose
+
+  creates symbolic link dir/file1, dir/file2 ... which point to
+  file1, file2 ... If DIR is not a directory, raises Errno::ENOTDIR.
+  If last argument is a directory, links DIR/LIST[0] to LIST[0],
+  DIR/LIST[1] to LIST[1], ....
+  creates symbolic links DESTDIR/LIST[0] which points LIST[0].
+  DESTDIR/LIST[1] to LIST[1], ....
+  If DESTDIR is not a directory, raises Errno::ENOTDIR.
+
+   FileUtils.ln_s Dir.glob('bin/*.rb'), '/home/aamine/bin'
+
+--- FileUtils.ln_sf( src, dest, *options )
+  Options: noop verbose
+
+  same to ln_s(src,dest,:force)
+
+--- FileUtils.cp( src, dest, *options )
+  Options: preserve noop verbose
+
+  copies a file SRC to DEST. If DEST is a directory, copies
+  SRC to DEST/SRC.
+
+    FileUtils.cp 'eval.c', 'eval.c.org'
+
+--- FileUtils.cp( list, dir, *options )
+  Options: preserve noop verbose
+
+  copies FILE1 to DIR/FILE1, FILE2 to DIR/FILE2 ...
+
+    FileUtils.cp 'cgi.rb', 'complex.rb', 'date.rb', '/usr/lib/ruby/1.6'
+    FileUtils.cp :verbose, %w(cgi.rb complex.rb date.rb), '/usr/lib/ruby/1.6'
+
+--- FileUtils.cp_r( src, dest, *options )
+  Options: preserve noop verbose
+
+  copies SRC to DEST. If SRC is a directory, this method copies
+  its all contents recursively. If DEST is a directory, copies
+  SRC to DEST/SRC.
+
+    # installing ruby library "mylib" under the site_ruby
+    FileUtils.rm_r site_ruby + '/mylib', :force
+    FileUtils.cp_r 'lib/', site_ruby + '/mylib'
+
+--- FileUtils.cp_r( list, dir, *options )
+  Options: preserve noop verbose
+
+  copies a file or a directory LIST[0] to DIR/LIST[0], LIST[1] to DIR/LIST[1], ...
+  If LIST[n] is a directory, copies its contents recursively.
+
+    FileUtils.cp_r %w(mail.rb field.rb debug/) site_ruby + '/tmail'
+    FileUtils.cp_r Dir.glob('*.rb'), '/home/aamine/lib/ruby', :noop, :verbose
+
+--- FileUtils.mv( src, dest, *options )
+  Options: noop verbose
+
+  moves a file SRC to DEST.
+  If FILE and DEST exist on the different disk partition,
+  copies it.
+
+    FileUtils.mv 'badname.rb', 'goodname.rb'
+    FileUtils.mv 'stuff.rb', 'lib/ruby', :force
+
+--- FileUtils.mv( list, dir, *options )
+  Options: noop verbose
+
+  moves FILE1 to DIR/FILE1, FILE2 to DIR/FILE2 ...
+  If FILE and DEST exist on the different disk partition,
+  copies it.
+
+    FileUtils.mv 'junk.txt', 'dust.txt', '/home/aamine/.trash/'
+    FileUtils.mv Dir.glob('test*.rb'), 'T', :noop, :verbose
+
+--- FileUtils.rm( list, *options )
+  Options: force noop verbose
+
+  remove files LIST[0], LIST[1]... This method cannot remove directory.
+  This method ignores all errors when :force option is set.
+
+    FileUtils.rm %w( junk.txt dust.txt )
+    FileUtils.rm Dir['*.so']
+    FileUtils.rm 'NotExistFile', :force    # never raises exception
+
+--- FileUtils.rm_r( list, *options )
+  Options: force noop verbose
+
+  remove files LIST[0] LIST[1]... If LIST[n] is a directory,
+  removes its all contents recursively. This method ignores
+  StandardError when :force option is set.
+
+    FileUtils.rm_r Dir.glob('/tmp/*')
+    FileUtils.rm_r '/', :force          #  :-)
+
+--- FileUtils.rm_rf( list, *options )
+  Options: noop verbose
+
+  same to rm_r(list,:force)
+
+--- FileUtils.cmp( file_a, file_b, *options )
+  Options: verbose
+
+  returns true if contents of a file A and a file B is identical.
+
+    FileUtils.cmp 'somefile', 'somefile'  #=> true
+    FileUtils.cmp '/bin/cp', '/bin/mv'    #=> maybe false.
+
+--- FileUtils.install( src, dest, mode = <src's>, *options )
+  Options: noop verbose
+
+  If SRC is not same to DEST, copies it and changes the permittion
+  mode to MODE.
+
+    FileUtils.install 'ruby', '/usr/local/bin/ruby', 0755, :verbose
+    FileUtils.install 'lib.rb', '/usr/local/lib/ruby/site_ruby', :verbose
+
+--- FileUtils.chmod( mode, list, *options )
+  Options: noop verbose
+
+  changes permittion bits on the named FILEs to the bit pattern
+  represented by MODE.
+
+    FileUtils.chmod 0644, 'my.rb', 'your.rb'
+    FileUtils.chmod 0755, 'somecommand'
+    FileUtils.chmod 0755, '/usr/bin/ruby', :verbose
+
+--- FileUtils.touch( list, *options )
+  Options: noop verbose
+
+  updates modification time (mtime) and access time (atime) of
+  LIST[0], LIST[1]...
+  If LIST[n] does not exist, creates an empty file.
+
+    FileUtils.touch 'timestamp'
+    FileUtils.touch Dir.glob('*.c');  system 'make'
+
+== module FileUtils::Verbose
+
+This class has all methods of FileUtils module and it works as
+same, but outputs messages before action. You can also pass
+verbose flag to all methods.
+
+== module FileUtils::NoWrite
+
+This class has all methods of FileUtils module,
+but never changes files/directories.
+
+=end
 
 
 module FileUtils
@@ -21,7 +269,7 @@ module FileUtils
     return false unless FileTest.exist? new
     new_time = File.ctime(new)
     args.each do |old|
-      if FileTest.exist? old then
+      if FileTest.exist? old
         return false unless new_time > File.mtime(old)
       end
     end
@@ -46,14 +294,14 @@ module FileUtils
     fu_output_message "mkdir -p #{list.join ' '}" if verbose
     return *list if noop
 
-    list.collect {|n| File.expand_path(n) }.each do |dir|
+    list.map {|n| File.expand_path(n) }.each do |dir|
       stack = []
-      until FileTest.directory? dir do
+      until FileTest.directory? dir
         stack.push dir
         dir = File.dirname(dir)
       end
-      stack.reverse_each do |dir|
-        Dir.mkdir dir
+      stack.reverse_each do |n|
+        Dir.mkdir n
       end
     end
 
@@ -81,7 +329,7 @@ module FileUtils
     fu_output_message "ln #{argv.join ' '}" if verbose
     return if noop
 
-    fu_each_src_dest( src, dest ) do |s,d|
+    fu_each_src_dest(src, dest) do |s,d|
       remove_file d, true if force
       File.link s, d
     end
@@ -94,7 +342,7 @@ module FileUtils
     fu_output_message "ln -s#{force ? 'f' : ''} #{[src,dest].flatten.join ' '}" if verbose
     return if noop
 
-    fu_each_src_dest( src, dest ) do |s,d|
+    fu_each_src_dest(src, dest) do |s,d|
       remove_file d, true if force
       File.symlink s, d
     end
@@ -113,7 +361,7 @@ module FileUtils
     fu_output_message "cp#{preserve ? ' -p' : ''} #{[src,dest].flatten.join ' '}" if verbose
     return if noop
 
-    fu_each_src_dest( src, dest ) do |s,d|
+    fu_each_src_dest(src, dest) do |s,d|
       fu_preserve_attr(preserve, s, d) {
           copy_file s, d
       }
@@ -127,8 +375,8 @@ module FileUtils
     fu_output_message "cp -r #{[src,dest].flatten.join ' '}" if verbose
     return if noop
 
-    fu_each_src_dest( src, dest ) do |s,d|
-      if FileTest.directory? s then
+    fu_each_src_dest(src, dest) do |s,d|
+      if FileTest.directory? s
         fu_copy_dir s, d, '.', preserve
       else
         fu_p_copy s, d, preserve
@@ -143,7 +391,7 @@ module FileUtils
         Dir.mkdir dir unless FileTest.directory? dir
     }
     Dir.entries( "#{src}/#{rel}" ).each do |fn|
-      if FileTest.directory? File.join(src,rel,fn) then
+      if FileTest.directory? File.join(src,rel,fn)
         next if /\A\.\.?\z/ === fn
         fu_copy_dir src, dest, "#{rel}/#{fn}", preserve
       else
@@ -161,7 +409,7 @@ module FileUtils
   private :fu_p_copy
 
   def fu_preserve_attr( really, src, dest )
-    unless really then
+    unless really
       yield src, dest
       return
     end
@@ -170,7 +418,7 @@ module FileUtils
     yield src, dest
     File.utime st.atime, st.mtime, dest
     begin
-      File.chown st.uid, st.gid
+      File.chown st.uid, st.gid, dest
     rescue Errno::EPERM
       File.chmod st.mode & 01777, dest   # clear setuid/setgid
     else
@@ -180,14 +428,12 @@ module FileUtils
   private :fu_preserve_attr
 
   def copy_file( src, dest )
-    st = r = w = nil
-
-    File.open( src,  'rb' ) {|r|
-    File.open( dest, 'wb' ) {|w|
-        st = r.stat
+    bsize = fu_blksize(File.stat(src).blksize)
+    File.open(src,  'rb') {|r|
+    File.open(dest, 'wb') {|w|
         begin
-          while true do
-            w.write r.sysread(st.blksize)
+          while true
+            w.syswrite r.sysread(bsize)
           end
         rescue EOFError
         end
@@ -200,16 +446,15 @@ module FileUtils
     fu_output_message "mv #{[src,dest].flatten.join ' '}" if verbose
     return if noop
 
-    fu_each_src_dest( src, dest ) do |s,d|
-      if /djgpp|cygwin|mswin32/ === RUBY_PLATFORM and
-         FileTest.file? d then
-         File.unlink d
+    fu_each_src_dest(src, dest) do |s,d|
+      if cannot_overwrite_file? and FileTest.file? d
+        File.unlink d
       end
 
       begin
         File.rename s, d
       rescue
-        if FileTest.symlink? s then
+        if FileTest.symlink? s
           File.symlink File.readlink(s), dest
           File.unlink s
         else
@@ -229,6 +474,10 @@ module FileUtils
 
   alias move mv
 
+  def cannot_overwrite_file?
+    /djgpp|cygwin|mswin32/ === RUBY_PLATFORM
+  end
+
 
   def rm( list, *options )
     force, noop, verbose, = fu_parseargs(options, :force, :noop, :verbose)
@@ -283,7 +532,7 @@ module FileUtils
     rescue
       # rescue dos?
       begin
-        if first then
+        if first
           first = false
           File.chmod 0777, fn
           retry
@@ -294,7 +543,7 @@ module FileUtils
   end
 
   def remove_dir( dir, force = false )
-    Dir.foreach( dir ) do |file|
+    Dir.foreach(dir) do |file|
       next if /\A\.\.?\z/ === file
       path = "#{dir}/#{file}"
       if FileTest.directory? path then remove_dir path, force
@@ -315,16 +564,17 @@ module FileUtils
 
     sa = sb = nil
     st = File.stat(filea)
+    bsize = fu_blksize(st.blksize)
     File.size(fileb) == st.size or return true
 
-    File.open( filea, 'rb' ) {|a|
-    File.open( fileb, 'rb' ) {|b|
+    File.open(filea, 'rb') {|a|
+    File.open(fileb, 'rb') {|b|
       begin
-        while sa == sb do
-          sa = a.read( st.blksize )
-          sb = b.read( st.blksize )
-          unless sa and sb then
-            if sa.nil? and sb.nil? then
+        while sa == sb
+          sa = a.read(bsize)
+          sb = b.read(bsize)
+          unless sa and sb
+            if sa.nil? and sb.nil?
               return true
             end
           end
@@ -345,7 +595,7 @@ module FileUtils
     return if noop
 
     fu_each_src_dest( src, dest ) do |s,d|
-      unless FileTest.exist? d and cmp(s,d) then
+      unless FileTest.exist? d and cmp(s,d)
         remove_file d, true
         copy_file s, d
         File.chmod mode, d if mode
@@ -383,10 +633,10 @@ module FileUtils
 
   def fu_parseargs( opts, *flagdecl )
     tab = {}
-    if opts.last == true or opts.last == false then
+    if opts.last == true or opts.last == false
       tab[:verbose] = opts.pop
     end
-    while Symbol === opts.last do
+    while Symbol === opts.last
       tab[opts.pop] = true
     end
 
@@ -402,7 +652,7 @@ module FileUtils
   end
 
   def fu_each_src_dest( src, dest )
-    unless Array === src then
+    unless Array === src
       yield src, fu_dest_filename(src, dest)
     else
       dir = dest
@@ -415,13 +665,21 @@ module FileUtils
   end
 
   def fu_dest_filename( src, dest )
-    if FileTest.directory? dest then
+    if FileTest.directory? dest
       (dest[-1,1] == '/' ? dest : dest + '/') + File.basename(src)
     else
       dest
     end
   end
 
+  def fu_blksize( size )
+    if size.nil? or size.zero?
+      2048
+    else
+      size
+    end
+  end
+
 
   @fileutils_output = $stderr
   @fileutils_label  = 'fileutils.'
@@ -481,11 +739,11 @@ module FileUtils
       next unless opts.include? 'verbose'
       module_eval <<-End, __FILE__, __LINE__ + 1
           def #{name}( *args )
-            unless defined? @fileutils_verbose then
+            unless defined? @fileutils_verbose
               @fileutils_verbose = true
             end
             args.push :verbose if @fileutils_verbose
-            super( *args )
+            super(*args)
           end
       End
     end
@@ -507,11 +765,11 @@ module FileUtils
       next unless opts.include? 'noop'
       module_eval <<-End, __FILE__, __LINE__ + 1
           def #{name}( *args )
-            unless defined? @fileutils_nowrite then
+            unless defined? @fileutils_nowrite
               @fileutils_nowrite = true
             end
             args.push :noop if @fileutils_nowrite
-            super( *args )
+            super(*args)
           end
       End
     end
@@ -542,7 +800,7 @@ module FileUtils
       module_eval <<-End, __FILE__, __LINE__ + 1
           def #{name}( *args )
             #{s}
-            super( *args )
+            super(*args)
           end
       End
     end
