@@ -15,7 +15,7 @@ module TMail
 
   class Scanner_R
 
-    Version = '0.10.5'
+    Version = '0.10.6'
     Version.freeze
 
 
@@ -97,7 +97,7 @@ module TMail
     attr_accessor :debug
 
     def scan( &block )
-      if @debug then
+      if @debug
         scan_main do |arr|
           s, v = arr
           printf "%7d %-10s %s\n",
@@ -123,32 +123,32 @@ module TMail
     }
 
     def scan_main
-      until eof? do
-        if skip( /\A[\n\r\t ]+/n ) then   # LWSP
+      until eof?
+        if skip(/\A[\n\r\t ]+/n)   # LWSP
           break if eof?
         end
 
-        if s = readstr(@word_re) then
-          if @is_mime_header then
+        if s = readstr(@word_re)
+          if @is_mime_header
             yield :TOKEN, s
           else
             # atom
-            if /\A\d+\z/ === s then
+            if /\A\d+\z/ === s
               yield :DIGIT, s
-            elsif @received then
+            elsif @received
               yield RECV_TOKEN[s.downcase] || :ATOM, s
             else
               yield :ATOM, s
             end
           end
 
-        elsif skip( /\A"/ ) then
+        elsif skip(/\A"/)
           yield :QUOTED, scan_quoted_word()
 
-        elsif skip( /\A\[/ ) then
+        elsif skip(/\A\[/)
           yield :DOMLIT, scan_domain_literal()
 
-        elsif skip( /\A\(/ ) then
+        elsif skip(/\A\(/)
           @comments.push scan_comment()
 
         else
@@ -170,7 +170,7 @@ module TMail
 
     def scan_qstr( pattern, terminal, type )
       result = ''
-      until eof? do
+      until eof?
         if    s = readstr(pattern) then result << s
         elsif skip(terminal)       then return result
         elsif skip(/\A\\/)         then result << readchar()
@@ -186,7 +186,7 @@ module TMail
       nest = 1
       content = @comment_re
 
-      until eof? do
+      until eof?
         if s = readstr(content) then result << s
         elsif skip(/\A\)/)      then nest -= 1
                                      return result if nest == 0
@@ -217,7 +217,7 @@ module TMail
     end
 
     def readstr( re )
-      if m = re.match(@src) then
+      if m = re.match(@src)
         @src = m.post_match
         m[0]
       else
@@ -230,7 +230,7 @@ module TMail
     end
 
     def skip( re )
-      if m = re.match(@src) then
+      if m = re.match(@src)
         @src = m.post_match
         true
       else
